% Data reading from Dynacar
load('Vehicle.mat');
vehicle = oVehicle.signals.values;
T_full  = oVehicle.time;

% Input variables
Vx_atIMU_full =  + vehicle(:, 5);
Vy_atIMU_full =  + vehicle(:, 6);
Yaw_full      =  + vehicle(:, 9);
Omega_full    =  + vehicle(:,10);
Ax_atIMU_full =  + vehicle(:, 7);
Ay_atIMU_full =  + vehicle(:, 8);
Delta_full    =  - vehicle(:,12)*21*pi/180; % Angulo del volante en radian

% Configuration parameters
Vcutoff  = 12.000; % Data cutoff speed                         [m/s]                                          
mu       = 1.000; % Friction coefficient                      [~]
g        = 9.810; % Gravity                                   [m/s^2]
h        = 0.410; % IMU height in reference to the rear axis  [m]
L        = 1.686; % Wheelbase                                 [m]
k_smooth = 5.000; % Smoothing factor                          [~]

% Additional variables (Calculated)
Veh_V = sqrt( Vx_atIMU_full.^2 + Vx_atIMU_full.^2 ); % Vehicle instantenous speed
idx   = find( Veh_V > Vcutoff );                     % Indexes over cutoff speed
T     = T_full(idx) - T_full(idx(1));                % Starting time offset

% d(Yaw rate)/dt calculation
dOmega_full = [0;diff(Omega_full)./diff(T_full)];
Omega       = Omega_full(idx);
dOmega      = dOmega_full(idx);

% Speeds at COR and COF
Vx_atCOR_full = Vx_atIMU_full;                % Longitudinal speed on Center of rear axis
Vy_atCOR_full = Vy_atIMU_full;                % Lateral speed on Center of rear axis
Vx_atCOF_full = Vx_atIMU_full;                % Longitudinal speed on Center of front axis
Vy_atCOF_full = Vy_atCOR_full + Omega_full*L; % Lateral speed on Center of front axis

% Vehicle slip angle
Beta_atCOR_full = atan2(Vy_atCOR_full,Vx_atCOR_full); % Slip angle of the rear wheel
Beta_atCOF_full = atan2(Vy_atCOF_full,Vx_atCOF_full); % Slip angle of the front wheel
Beta_atCOR      = Beta_atCOR_full(idx);               % Slip angle of the rear wheel  (relevant indexes)
Beta_atCOF      = Beta_atCOF_full(idx);               % Slip angle of the front wheel (relevant indexes)

% Calculos con el angulo de volante
Delta       = Delta_full(idx);     % Wheel angle in radians
Alpha_atCOF = (Beta_atCOF-Delta); % 

Ax_atIMU = Ax_atIMU_full(idx);
Ay_atIMU = Ay_atIMU_full(idx);

% accelerations at COG
Ax = @(LR) Ax_atIMU - Omega.^2*(LR);
Ay = @(LR) Ay_atIMU + dOmega*(LR);

% Vertical forces
fzf = @(LR_L,ax) (   (LR_L).*g - h/L.*ax ); % units of [m/s^2]
fzr = @(LR_L,ax) ( (1-LR_L).*g + h/L.*ax ); % units of [m/s^2]

% Lateral forces
fyf = @(cf,LR_L,alphaf,ax) ( mu.*cf.*fzf(LR_L,ax).*alphaf ); % units of [m/s^2]
fyr = @(cr,LR_L,alphar,ax) ( mu.*cr.*fzr(LR_L,ax).*alphar ); % units of [m/s^2]

% Forces and moments balance                                               
ay_est     = @(J_m,LR_L,cf,cr,alphaf,betar,ax)                   fyf(cf,LR_L,alphaf,ax) +         fyr(cr,LR_L,betar,ax)  ; % units of [m/s^2]                         
dOmega_est = @(J_m,LR_L,cf,cr,alphaf,betar,ax) L/J_m*( (1-LR_L).*fyf(cf,LR_L,alphaf,ax) - (LR_L).*fyr(cr,LR_L,betar,ax) ); % units of [rad/s^2]

e_ay = @(p)Ay(p(2)*L)  -  ay_est(p(1),p(2),p(3),p(4),Alpha_atCOF,Beta_atCOR,Ax(p(2)*L));
e_domega = @(p)dOmega  -  dOmega_est(p(1),p(2),p(3),p(4),Alpha_atCOF,Beta_atCOR,Ax(p(2)*L));

fa = @(p) sum(e_ay(p).^2 + e_domega(p).^2);

% tire force based comparison
fyf_m = @(J_m,LR_L,dOmega,ay)( dOmega +    LR_L *L/J_m*ay)*J_m/L;
fyr_m = @(J_m,LR_L,dOmega,ay)(-dOmega + (1-LR_L)*L/J_m*ay)*J_m/L;

e_fyf = @(p)fyf(p(3),p(2),Alpha_atCOF,Ax(p(2)*L)) - fyf_m(p(1),p(2), dOmega,Ay(p(2)*L));
e_fyr = @(p)fyr(p(4),p(2),Beta_atCOR ,Ax(p(2)*L)) - fyr_m(p(1),p(2), dOmega,Ay(p(2)*L));

ff = @(p) sum(e_fyf(p).^2 + e_fyr(p).^2);
fx = @(p) ff(p) + fa(p);
p  = [0.4513, 0.7811, -5, -5];

[p,cost] = fminsearch(fx,p,optimset('TolX',0.0001));% clc;

disp(['p1: J_m  =   ' num2str(round(p(1),4))]);
disp(['p2: LR_L =   ' num2str(round(p(2),4))]);
disp(['p3: cf   = '   num2str(round(p(3),4))]);
disp(['p4: cr   = '   num2str(round(p(4),4))]);
disp(' ');



figure
subplot(2,1,1);
hold all;
box on;
plot(Ay(p(2)*L));
plot(ay_est(p(1),p(2),p(3),p(4),Alpha_atCOF,Beta_atCOR,Ax(p(2)*L)));
legend('measured','model');
ylabel('a_y^{COG} [m/s^2]');
axis tight;

subplot(2,1,2);
hold all;
box on;
plot(dOmega);
plot(dOmega_est(p(1),p(2),p(3),p(4),Alpha_atCOF,Beta_atCOR,Ax(p(2)*L)));
xlabel('datapoints');
ylabel('d\omega/dt [\circ/s^2]');
axis tight;



figure;
subplot(2,1,1);hold all;
x_base = Alpha_atCOF*180/pi;
y_base = fyf_m(p(1),p(2),dOmega,Ay(p(2)*L))./fzf(p(2),Ax(p(2)*L));
plot(x_base,y_base,'Marker','.','LineStyle','none');
x_base = [min(x_base),max(x_base)]*1.3;
plot(x_base,p(3)*(x_base/180*pi),'LineWidth',2,'Color','red');
xlabel('\alpha_f [\circ]');
ylabel('F_{yf}/F_{zf} [m/s²]');
axis tight;

subplot(2,1,2);hold all;
x_base = Beta_atCOR*180/pi;
y_base = fyr_m(p(1),p(2),dOmega,Ay(p(2)*L))./fzr(p(2),Ax(p(2)*L)); 
plot(x_base,y_base,'Marker','.','LineStyle','none');
x_base = [min(x_base),max(x_base)]*1.5;
plot(x_base,p(4)*(x_base/180*pi),'LineWidth',2,'Color','red');
xlabel('\alpha_r [\circ]');
ylabel('F_{yr}/F_{zr} [m/s²]');
axis tight;
%clear all;